unit TableServerOLEClass;

interface

uses
  OleAuto, OLE2;

type
  TTableProxy = class(TAutoObject)
  private
    function GetActive: WordBool;
    function GetDatabaseName: String;
    function GetTableData: Variant;
    function GetTableName: String;
    procedure SetActive(Value: WordBool);
    procedure SetDatabaseName(const Value: String);
    procedure SetTableName(const Value: String);
  automated
    procedure Close;
    procedure Open;
    property Active: WordBool read GetActive write SetActive;
    property DatabaseName: String read GetDatabaseName write SetDatabaseName;
    property TableData: Variant read GetTableData dispid DispID_Value;
    property TableName: String read GetTableName write SetTableName;
  end;

implementation

uses
  TableServerDataModule, Forms, Controls, DB;

function TTableProxy.GetActive: WordBool;
begin
  Result := DM.TblFrom.Active
end;

function TTableProxy.GetDatabaseName: String;
begin
  Result := DM.TblFrom.DatabaseName
end;

function TTableProxy.GetTableData: Variant;
var
  OuterLoop, InnerLoop: Integer;
begin
  with DM.TblFrom do
  begin
    //Make an array with one row more than the table has
    //so we can pass the field names as well
    Result := VarArrayCreate([0, FieldCount, 0, RecordCount], varOleStr);
    //Package up the field names
    for InnerLoop := 0 to FieldCount - 1 do
      Result[InnerLoop, 0] := Fields[InnerLoop].DisplayLabel;
    //Now package up the field contents
    DisableControls;
    First;
    Screen.Cursor := crHourglass;
    try
      for OuterLoop := 1 to RecordCount do
      begin
        for InnerLoop := 0 to FieldCount - 1 do
        begin
          Result[InnerLoop, OuterLoop] := Fields[InnerLoop].AsString;
          if Fields[InnerLoop] is TGraphicField then
            Result[InnerLoop, OuterLoop] := '(Graphic)';
        end;
        Next;
      end
    finally
      Screen.Cursor := crDefault;
      EnableControls
    end;
  end;
end;

function TTableProxy.GetTableName: String;
begin
  Result := DM.TblFrom.TableName
end;

procedure TTableProxy.SetActive(Value: WordBool);
begin
  DM.TblFrom.Active := Value
end;

procedure TTableProxy.SetDatabaseName(const Value: String);
begin
  DM.TblFrom.DatabaseName := Value
end;

procedure TTableProxy.SetTableName(const Value: String);
begin
  DM.TblFrom.TableName := Value
end;

procedure TTableProxy.Close;
begin
  Active := False
end;

procedure TTableProxy.Open;
begin
  Active := True
end;

procedure RegisterTableProxy;
const
  AutoClassInfo: TAutoClassInfo = (
    AutoClass: TTableProxy;
    ProgID: 'TableServer.TableProxy';
    ClassID: '{456A4860-8749-11D0-9FEA-A42B00C10000}';
    Description: 'BDE Table Server';
    Instancing: acMultiInstance);
begin
  Automation.RegisterClass(AutoClassInfo);
end;

initialization
  RegisterTableProxy;
end.
