unit MutexUnit;

interface

uses
  Classes, MainFormUnit;

type
  TTestMutex = class(TThread)
  private
    FUIString: String;
    //Two routines to simplify showing our progress
    procedure UpdateUI;
    procedure WriteString(const S: String);
  protected
    //Body of the thread
    procedure Execute; override;
    //This relies on mutually exclusive access
    procedure ThisNeedsMutualExclusiveAccess;
  public
    constructor Create;
    destructor Destroy; override;
  end;

implementation

uses
  SysUtils, Windows;

var
  Mutex: THandle;

const
  MutexName = 'SampleMutex';

constructor TTestMutex.Create;
begin
  inherited Create(False);
  WriteString(Format('Thread %x started', [Handle]))
end;

destructor TTestMutex.Destroy;
begin
  WriteString(Format('Thread %x terminated', [Handle]));
  inherited Destroy
end;

procedure TTestMutex.UpdateUI;
begin
  Form1.Listbox1.Items.Add(FUIString)
end;

procedure TTestMutex.WriteString(const S: String);
begin
  FUIString := S;
  Synchronize(UpdateUI);
end;

procedure TTestMutex.Execute;
begin
  WriteString(Format(
    'Thread %x waiting for mutex', [Handle]));
  WaitForSingleObject(Mutex, Infinite);
  try
    ThisNeedsMutualExclusiveAccess
  finally
    ReleaseMutex(Mutex)
  end
end;

procedure TTestMutex.ThisNeedsMutualExclusiveAccess;
begin
  WriteString(Format(
    'Thread %x got mutex', [Handle]));
  Sleep(4000);
  WriteString(Format(
    'Thread %x finished with mutex', [Handle]));
end;

initialization
  Mutex := CreateMutex(nil, False, MutexName)
finalization
  //This is not strictly necessary as all open
  //handles are closed by the Win32 OS upon termination
  if Mutex <> 0 then
    CloseHandle(Mutex)
end.