unit SysTrays;
//Written by Roy Nelson from Borland
//This code is not thoroughly tested but
//should give you a starting point

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ShellApi, Dialogs;

const
  wm_TrayCallback = wm_App + 111; //Custom message

type
  TWMTrayCallback = record
    Msg: Cardinal;
    uID: Word;
    MouseMessage: LongInt;
    Result: Longint;
  end;

  TSysTray = class(TGraphicControl)
  private
    tnid              : TNotifyIconData;
    ApplicationHandle : HWND;
    FIcon             : TIcon;
    FMyShowHint       : Boolean;
    FCursor           : TCursor;
    FReadOnlyTag      : LongInt;
    FShutdown         : Boolean;
    FOldFormOnCloseQuery  : TCloseQueryEvent;
    FOnCloseQuery         : TCloseQueryEvent;
    procedure RefreshIcon(Sender: TObject);
    //Follow MS's lead I for Internal
    procedure ISetIcon(Value: TIcon);
    procedure ISetShowHint(Value: Boolean);
    procedure ISetHint(const Value: string);
    procedure IISetVisible(Value: Boolean);
    procedure ISetVisible(Value: Boolean);
    function  IVisible : boolean;
    procedure CompleteHide(Value: Boolean);
    procedure EmptyNotifyIconData;
    function  IGetWidth  : integer;
    function  IGetHeight : integer;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure RemoveFromTray;
    procedure ISetShutDown(Value : boolean);
  protected
    procedure Paint; override;
    procedure Loaded; override;
    //prevent the component's being resized in the designer
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    function  Tray_CallbackProc(var Msg: TMessage): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    property Shutdown : Boolean read FShutdown write ISetShutDown default false;
  published
    property Width  : Integer read IGetWidth;
    property Height : Integer read IGetHeight;
    property Cursor : TCursor read FCursor;
    property Tag    : Longint read FReadOnlyTag default 200;
    property Hint write ISetHint;
    property PopupMenu;
    property ShowHint read FMyShowHint write ISetShowHint stored true;
    property Visible : Boolean read IVisible write ISetVisible default true;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property Icon: TIcon read FIcon write ISetIcon stored true;
    property OnCloseQuery: TCloseQueryEvent
      read FOnCloseQuery write FOnCloseQuery;
  end;

procedure Register;

implementation

procedure TSysTray.ISetShutDown(Value : boolean);
begin
  if FShutDown <> Value then
    FShutDown := Value;
  if FShutDown then
    RemoveFromTray;
end;

procedure TSysTray.RemoveFromTray;
begin
  FIcon.Free;
  EmptyNotifyIconData;
  tnid.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
  Shell_NotifyIcon(NIM_DELETE, @tnid);
end;

function TSysTray.IGetWidth  : integer;
begin
  Result := inherited Width;
end;

function TSysTray.IGetHeight : integer;
begin
  Result := inherited Height;
end;

procedure TSysTray.ISetIcon(Value: TIcon);
begin
  FIcon.Assign(Value);
  EmptyNotifyIconData;
  tnid.hIcon  := FIcon.Handle;
  tnid.uFlags := NIF_ICON;
  Shell_NotifyIcon(NIM_MODIFY, @tnid);
  Refresh;
end;


function TSysTray.IVisible : boolean;
begin
  Result := inherited Visible;
end;

procedure TSysTray.IISetVisible(Value: Boolean);
begin
  inherited Visible := Value;
end;

procedure TSysTray.CompleteHide(Value: Boolean);
const
  ShowOrHide :array [Boolean] of Integer = (SW_HIDE, SW_SHOWNORMAL);
  //Use Brian's trick to overcome the pascal's lack of ? : operator
begin
  //We do not want the form to disappear when designing
  if csDesigning in ComponentState  then
    Exit;
  //This will Remove the Application's Icon from the taskbar
  Application.ShowMainForm := Value;
{$ifdef Ver90}
  //This trick is not needed for Delphi 3+
  ShowWindow(ApplicationHandle, ShowOrHide[Value]);
{$endif}
  ShowWindow(Parent.Handle, ShowOrHide[Value]);
end;

procedure TSysTray.ISetVisible(Value: Boolean);
var
  aSysTray : TComponent;
  i        : integer;
begin
  //If one changes we all do but only the main
  //200 component is allowed to hide or show the form
  for i := 0 to Parent.ComponentCount-1 do
  begin
    aSysTray := Parent.Components[i];
    if (aSysTray is TSysTray) then
      if 200 <> TSysTray(aSysTray).FReadOnlyTag then
        TSysTray(aSysTray).IISetVisible(Value)
      else
      begin
        CompleteHide(Value);
        TSysTray(aSysTray).IISetVisible(Value);
      end;
  end;
end;

function TSysTray.Tray_CallbackProc(var Msg: TMessage): boolean;
var
  P :TPoint;
begin
  Result := False;
  //We have to do this in case there are messages pending
  //for this icon then we bale out
  if csDestroying in ComponentState then
    Exit;
  if (Msg.Msg = WM_TRAYCALLBACK) then
    with TWMTrayCallback(Msg) do
      if uId = FReadOnlyTag then
      begin
        P := Point(1,1);
        if (MouseMessage = WM_RButtonUp) and
           (Assigned(PopUpMenu)) then
        begin
          GetCursorPos(P);
          //This will lie to the VCL in the systray about the cursor
          //position, so that the menu can popup at the right position
          P:= Parent.ScreenToClient(P);
        end;
        SetForegroundWindow(Parent.Handle);
        Perform(MouseMessage,0,Longint(PointToSmallPoint(P)));
      end;
end;

procedure TSysTray.Paint;
begin
  if (csDesigning in ComponentState) then
    Canvas.Draw(0, 0, FIcon);
end;

procedure TSysTray.ISetShowHint(Value: Boolean);
const
  ShowOrHideHint :
    array [Boolean] of integer = (NIM_DELETE, NIM_MODIFY);
  //Brian's trick again
begin
  if FMyShowHint <> Value then
  begin
    FMyShowHint := Value;
    if not (csReading in ComponentState) then
    begin
      EmptyNotifyIconData;
      StrLCopy(tnid.szTip, PChar(Hint),sizeof(tnid.szTip)-1);
      tnid.uFlags :=  NIF_TIP;
      Shell_NotifyIcon(ShowOrHideHint[Value], @tnid);
    end;
  end;
end;

procedure TSysTray.EmptyNotifyIconData;
begin
  with tnid do
  begin
    cbSize := sizeof(TNotifyIconData);
    Wnd    := ApplicationHandle;
    uID    := Tag;
    uFlags := 0;
    uCallbackMessage := 0;
    hIcon  := 0;
    szTip  := #0;
 end;
end;

procedure TSysTray.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if (csDesigning in ComponentState) then
    inherited SetBounds(ALeft, ATop, 32, 32)
  else
    inherited SetBounds(0, 0, 2, 2);
end;

procedure TSysTray.Loaded;
begin
  if not (csDesigning in ComponentState)  then
  begin
    EmptyNotifyIconData;
    if ShowHint then
    begin
      StrLCopy(tnid.szTip, PChar(Hint),sizeof(tnid.szTip)-1);
      tnid.uFlags := NIF_TIP;
    end;
    tnid.uCallbackMessage := WM_TRAYCALLBACK;
    tnid.hIcon            := Icon.Handle;
    tnid.uFlags           := tnid.uFlags or NIF_ICON or NIF_MESSAGE;
    Shell_NotifyIcon(NIM_ADD, @tnid);
    //Look at the main Window messages
    Application.HookMainWindow(Tray_CallbackProc);
  end;
  inherited Loaded;
end;

procedure TSysTray.RefreshIcon(Sender: TObject);
begin
  EmptyNotifyIconData;
  tnid.uFlags :=  NIF_ICON;
  Shell_NotifyIcon(NIM_MODIFY, @tnid);
end;

constructor TSysTray.Create(AOwner: TComponent);
var
  aSysTray : TComponent;
  i : integer;
begin
  inherited Create(AOwner);
  if (Owner is TForm) then
  begin
    Parent        := TForm(Owner);
    //Store the handles to hide and show
    ApplicationHandle  := Application.Handle;
    FCursor            := crDefault;
    Enabled            := true;
    inherited Width    := 32;
    inherited Height   := 32;
    FMyShowHint        := false;
    FShutDown          := false;
    FIcon              := TIcon.Create;
    FIcon.Onchange     := ReFreshIcon;
    if Assigned(TForm(Owner).OnCloseQuery) then
       FOldFormOnCloseQuery := TForm(Owner).OnCloseQuery;
    TForm(Owner).OnCloseQuery := FormCloseQuery;
    //Use our own tag to keep track of tags and Automatically
    //create unique tags
    FReadOnlyTag := 200;
    for i := 0 to Parent.ComponentCount-1 do
    begin
      aSysTray := Parent.Components[i];
      if (aSysTray is TSysTray) and
        (aSysTray <> Self) then
        if FReadOnlyTag <= TSysTray(aSysTray).FReadOnlyTag then
           FReadOnlyTag := TSysTray(aSysTray).FReadOnlyTag + 1;
    end;
  end
  else
    Abort;
end;

destructor TSysTray.Destroy;
begin
  //Only remove the Hook if is has not already been done
  if not FShutDown then
    RemoveFromTray;
  inherited Destroy;
end;

procedure TSysTray.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
 AllClose : boolean;
begin
  AllClose := CanClose;
  //only need to call our own canclose when we have not shutdown yet
  if Assigned(FOnCloseQuery) and not FShutDown then
     FOnCloseQuery(Sender,CanClose);
  AllClose := CanClose and AllClose;
  //Chain it on
  if Assigned(FOldFormOnCloseQuery) then
     FOldFormOnCloseQuery(Sender,CanClose);
  AllClose := CanClose and AllClose;
  CanClose := AllClose;
end;

procedure TSysTray.ISetHint(const Value: string);
begin
   if inherited Hint <> Value then
   begin
     inherited Hint := Value;
     EmptyNotifyIconData;
     StrLCopy(tnid.szTip, PChar(Hint),sizeof(tnid.szTip)-1);
     tnid.uFlags :=  NIF_TIP;
     Shell_NotifyIcon(NIM_MODIFY, @tnid);
   end;
end;

procedure Register;
begin
  RegisterComponents('DDG', [TSysTray]);
end;

end.
