unit DLLWizU;

interface

uses
{$ifdef Win32}
  ShareMem, //32-bit DLL-specific
{$endif}
  ToolIntf, VirtIntf,
  ExptIntf, WinProcs, WinTypes, SysUtils;

type
  TSampleExpert = class(TIExpert)
  private
    FStyle: TExpertStyle;
  public
    constructor Create;
    destructor Destroy; override;
    function GetName:        string; override;
    function GetComment:     string; override;
{$ifdef Win32}
    function GetAuthor:      string; override;
    function GetPage:        string; override;
    function GetGlyph:       HICON;  override;
{$else}
    function GetGlyph:       HBITMAP;  override;
{$endif}
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString:    string; override;
    function GetMenuText:    string; override;
    procedure Execute;               override;
  end;

function InitExpert(ToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean;
  export; {$ifdef Win32}stdcall;{$endif}

{$ifndef Win32}
function FaultHandler(FaultID: Word; faultAddr: Pointer): TFaultResponse; export;
{$endif}

implementation

uses
  Dialogs;

procedure HandleException;
begin
  if Assigned(ToolServices) then
    ToolServices.RaiseException(ReleaseException)
end;

constructor TSampleExpert.Create;
begin
  try
    inherited Create;
    Randomize;
    FStyle := TExpertStyle(Random(Integer(High(TExpertStyle))+1));
  except
    HandleException
  end
end;

destructor TSampleExpert.Destroy;
begin
  inherited Destroy
end;

function TSampleExpert.GetName: string;
begin
  try
    Result := 'Brian''s Sample DLL Wizard';
  except
    HandleException
  end
end;

function TSampleExpert.GetComment: string;
begin
  try
    Result := '';
    if GetStyle in [esForm, esProject] then
      Result := 'This is where I say what this wizard does';
  except
    HandleException
  end
end ;

{$ifdef Win32}
function TSampleExpert.GetAuthor: string;
begin
  try
    Result := '';
    if GetStyle in [esForm, esProject] then
      Result := 'Brian Long';
  except
    HandleException
  end
end ;

function TSampleExpert.GetPage: string;
begin
  try
    Result := '';
    if GetStyle = esForm then
      Result := 'Forms';
    if GetStyle = esProject then
      Result := 'Projects';
  except
    HandleException
  end
end;

function TSampleExpert.GetGlyph: HICON;
begin
  Result := 0;
  try
    if GetStyle in [esForm, esProject] then
      Result := LoadIcon(0, IDI_HAND);
  except
    HandleException
  end
end;
{$else}
function TSampleExpert.GetGlyph: HBITMAP;
begin
  Result := 0;
  try
    if GetStyle in [esForm, esProject] then
      Result := LoadBitmap(0, PChar(OBM_CLOSE));
  except
    HandleException
  end
end;
{$endif}

{ We will have a random expert style }
function TSampleExpert.GetStyle: TExpertStyle;
begin
  try
{    Result := esStandard; }
{    Result := esForm;}
{    Result := esProject; }
{    Result := esAddIn; // Win32 only }
    Result := FStyle
  except
    HandleException
  end
end;

function TSampleExpert.GetState: TExpertState;
begin
  try
    Result := [esEnabled];
  except
    HandleException
  end
end;

function TSampleExpert.GetIDString: string;
begin
  try
    Result := 'Oblong.SampleDLLWizard';
  except
    HandleException
  end
end;

function TSampleExpert.GetMenuText: string;
begin
  try
    Result := '';
    if GetStyle = esStandard then
      Result := '&Brian''s Sample DLL Wizard'
  except
    HandleException
  end
end ;

procedure TSampleExpert.Execute;
begin
  try
  {$ifdef Win32}
    if GetStyle <> esAddIn then
  {$endif}
      ShowMessage('Abracadabra')
  except
    HandleException
  end
end;

procedure DoneExpert; export;
begin
  { Put any general destruction code here.  Note that the Delphi IDE }
  { will destroy any experts which have been registered. }
end;

function InitExpert(ToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean;
begin
  Result := False;
  try
    if ExptIntf.ToolServices = nil then
      ExptIntf.ToolServices := ToolServices; { Save! }
    { If you use the Forms unit, then use this line: }
    { if Assigned(ToolServices) then }
    {   Application.Handle := ToolServices.GetParentHandle; }
    Terminate := DoneExpert;
    if (@RegisterProc <> nil) then
      Result := RegisterProc(TSampleExpert.Create)
    { Do any important initialisation required by your wizard }
  except
    HandleException
  end
end;

{$ifndef Win32}
function FaultHandler(FaultID: Word; faultAddr: Pointer): TFaultResponse;
begin
  DefaultExceptHandler(FaultId, faultAddr);
end;
{$endif}

end.
