unit UnitWizU2;

interface

uses
   ExptIntf, ToolIntf, WinProcs, WinTypes, ExtCtrls;

type
  TSampleExpert = class(TIExpert)
  private
    FStyle: TExpertStyle;
  protected
    {$ifdef Win32}
    MenuItem: TIMenuItemIntf;
    procedure MenuOnClick(Sender: TIMenuItemIntf); virtual;
    {$endif}
  public
    constructor Create;
    destructor Destroy; override;
    function GetName:        string; override;
    function GetComment:     string; override;
{$ifdef Win32}
    function GetAuthor:      string; override;
    function GetPage:        string; override;
    function GetGlyph:       HICON;  override;
{$else}
    function GetGlyph:       HBITMAP;  override;
{$endif}
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString:    string; override;
    function GetMenuText:    string; override;
    procedure Execute;               override;
  end;

procedure Register;

implementation

uses
  Dialogs, Menus, Classes;

constructor TSampleExpert.Create;
{$ifdef Win32}
var
  Mainmenu: TIMainMenuIntf;
  Tools, ToolsGallery: TIMenuItemIntf;
{$endif}
begin
  inherited Create;
  Randomize;
  FStyle := TExpertStyle(Random(Integer(High(TExpertStyle))+1));
  { Set up the menu item(s) for an add-in wizard }
  {$ifdef Win32}
  if (GetStyle = esAddIn) and Assigned(ToolServices) then
  begin
    MainMenu := ToolServices.GetMainMenu;
    if Assigned(MainMenu) then
      try
        ToolsGallery := MainMenu.FindMenuItem('ToolsGalleryItem');
        if Assigned(ToolsGallery) then
          try
            Tools := ToolsGallery.GetParent;
            if Assigned(Tools) then
              try
                MenuItem := Tools.InsertItem(
                  ToolsGallery.GetIndex + 1,
                  'Brian''s Sample Add-In CompLib Wizard', '', '',
                  ShortCut(Ord('C'), [ssCtrl]), 0, 0,
                  [mfEnabled, mfVisible], MenuOnClick)
              finally
                Tools.DestroyMenuItem
              end
          finally
            ToolsGallery.DestroyMenuItem
          end
      finally
        MainMenu.Free
      end
  end
  {$endif}
end;

destructor TSampleExpert.Destroy;
begin
{$ifdef Win32}
  if Assigned(MenuItem) then
    MenuItem.DestroyMenuItem;
{$endif}
  inherited Destroy
end;

{$ifdef Win32}
procedure TSampleExpert.MenuOnClick(Sender: TIMenuItemIntf);
begin
  ShowMessage('Abracadabra')
end;
{$endif}

function TSampleExpert.GetName: string;
begin
  Result := 'Brian''s Sample CompLib Wizard';
end;

function TSampleExpert.GetComment: string;
begin
  Result := '';
  if GetStyle in [esForm, esProject] then
    Result := 'This is where I say what this wizard does';
end ;

{$ifdef Win32}
function TSampleExpert.GetAuthor: string;
begin
  Result := '';
  if GetStyle in [esForm, esProject] then
    Result := 'Brian Long';
end ;

function TSampleExpert.GetPage: string;
begin
  Result := '';
  if GetStyle = esForm then
    Result := 'Forms';
  if GetStyle = esProject then
    Result := 'Projects';
end;

function TSampleExpert.GetGlyph: HICON;
begin
  Result := 0;
  if GetStyle in [esForm, esProject] then
    Result := LoadIcon(0, IDI_HAND);
end;
{$else}
function TSampleExpert.GetGlyph: HBITMAP;
begin
  Result := 0;
  if GetStyle in [esForm, esProject] then
    Result := LoadBitmap(0, PChar(OBM_CLOSE));
end;
{$endif}

{ We will have a random expert style }
function TSampleExpert.GetStyle: TExpertStyle;
begin
{  Result := esStandard; }
{  Result := esForm;}
{  Result := esProject; }
  Result := esAddIn; // Win32 only 
{   Result := FStyle }
end;

function TSampleExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TSampleExpert.GetIDString: string;
begin
  Result := 'Oblong.SampleCompLibWizard';
end;

function TSampleExpert.GetMenuText: string;
begin
  Result := '';
  if GetStyle = esStandard then
    Result := '&Brian''s Sample CompLib Wizard'
end ;

procedure TSampleExpert.Execute;
begin
{$ifdef Win32}
  if GetStyle <> esAddIn then
{$endif}
    ShowMessage('Abracadabra')
end;

procedure Register;
begin
  RegisterLibraryExpert(TSampleExpert.Create)
end;

end.
